"use strict";(self.webpackChunk_aws_amazon_codeguru_extension=self.webpackChunk_aws_amazon_codeguru_extension||[]).push([[910],{9910:(o,n,t)=>{var e,a;function c(o,n){return Object.keys(o).some((function(t){return o[t]===n}))}function s(o,n,t){t?o.classList.add(n):o.classList.remove(n)}function i(o,n){void 0===n&&(n=document.body),!o||c(e,o)?s(n,"awsui-dark-mode",o===e.Dark):console.warn('Mode "'.concat(o,'" is not supported'))}function d(o,n){void 0===n&&(n=document.body),!o||c(a,o)?s(n,"awsui-compact-mode",o===a.Compact):console.warn('Density "'.concat(o,'" is not supported'))}function u(o,n){void 0===n&&(n=document.body),s(n,"awsui-motion-disabled",o)}t.r(n),t.d(n,{Density:()=>a,Mode:()=>e,applyDensity:()=>d,applyMode:()=>i,disableMotion:()=>u}),function(o){o.Light="light",o.Dark="dark"}(e||(e={})),function(o){o.Comfortable="comfortable",o.Compact="compact"}(a||(a={}))}}]);