import { LabIcon } from '@jupyterlab/ui-components';
import codeGuruIconSVG from '../../style/icons/cg-icon.svg';

const codeGuruIcon = new LabIcon({
  name: 'codeGuruIcon',
  svgstr: codeGuruIconSVG
});

export { codeGuruIcon };
